//*****************************************
// AVR ATmega644P Geiger Counter
// CPU Clock:8MHz,16NHz,18.432MHz or 20MHz(Compile Option)
// File Name:Comm2.c
// KCK[JE^[p
//   iq[Yf[^@is`fgpȂAȂAwIV[^(8MHz)Aɂj
//   i`FbN͂rohdmj
//
//  Use UART1
//  ̃KCK[JE^[łComm2(UART1)͎gpȂ
//
//  Use INT1 & Timer1
//  INT1:Start Bit Falling Edge
//  Timer1: Start, B0 -> B7, Parity & Stop Bit Center Timing
//
//  2011/09/02 New  Taishi ATmega644P(J308)
//  2011/10/22 Modify for LND712 by  Taishi
//  2011/10/28 Modify for SBM-20 by  Taishi
//  2011/11/08 Modify for LND712 by  Taishi
//  2011/11/12 Modify for SBM-20 by  Taishi
//  2011/11/12 Modify for J-308  by  Taishi
//  2011/11/24 Modify for All Model  Taishi
//*****************************************
#include	<avr/interrupt.h>
#include	<avr/io.h>

#include	"AVR_ATmega644_Geiger.h"
#include	"Serial.h"
#include	"LCD.h"
#include	"Utility.h"

#include	"Comm2.h"

#ifdef	NON_COMM2
#else	//NON_COMM2

// peB@t`qsP
#define	V_PARITY1_NON	((0 << UPM11) | (0 << UPM10))		// peB
#define	V_PARITY1_EVN	((1 << UPM11) | (0 << UPM10))		// peB
#define	V_PARITY1_ODD	((1 << UPM11) | (1 << UPM10))		// peB
unsigned int	v_parity1[CODE_PARITY_MAX+1] ={				// peBɑ΂ enum code_parity Ɠт̕KvL
						V_PARITY1_NON,						// peB
						V_PARITY1_EVN,						// peB
						V_PARITY1_ODD						// peB
				};

// f[^@t`qsP
#define	V_DATA1_8		((1 << UCSZ10) | (1 << UCSZ11))		// Wrbgf[^
#define	V_DATA1_7		((0 << UCSZ10) | (1 << UCSZ11))		// Vrbgf[^
unsigned int	v_data1_length[CODE_DATA_MAX+1] ={			// f[^ɑ΂ enum code_data_length Ɠт̕KvL
						V_DATA1_8,							// Wrbgf[^
						V_DATA1_7							// Vrbgf[^
				};

// tbrqPa̒l
#define	V_UCSR1B		((1 << RXEN1) | (1 << RXCIE1))		// tbrqPa̒l: MAM荞݋



volatile unsigned int	uart1_err_receiver;					// qwOG[ƎMf[^
volatile unsigned int	uart1_pass_time;					// oߎ

//***************************
// UART1 Disable
// CPU Clock:16MHz
//***************************
void	uart1_disable(void)
{
	UCSR1B = 0;
	write_data_pos = 0;													// ݎMf[^|WV
	data_get_run = CODE_DATA_GET_END;									// Mf[^擾I
	rxd_rec = 0;														// qwO͂qwPM
}
#ifdef		DEBUG_TX
//***************************
// UART1 Pre Initial
// CPU Clock:16MHz
//***************************
void	uart1_pre_init(void)
{
#ifdef	UART_BAISOKU													// t`qs̃{[[gpNbN{ݒɂ
	UCSR1A |= (1 << U2X1);												// {
#endif

	UBRR1  = v_ubrr[set_bps];											// aor

	UCSR1C = (v_parity1[set_parity] | v_data1_length[set_data_length]);	// peBAf[^

	UCSR1B = (1 << TXEN1);												// tbrqPa̒l: M

	write_data_pos = 0;													// ݎMf[^|WV
	data_get_run = CODE_DATA_GET_END;									// Mf[^擾I
	rxd_rec = 0;														// qwO͂qwPM
}
#endif		//DEBUG_TX
//***************************
// UART1 Initial
// CPU Clock:16MHz
//***************************
void	uart1_init(void)
{
#ifdef	UART_BAISOKU													// t`qs̃{[[gpNbN{ݒɂ
	UCSR1A |= (1 << U2X1);												// {
#endif

	UBRR1  = v_ubrr[set_bps];											// aor

	UCSR1C = (v_parity1[set_parity] | v_data1_length[set_data_length]);	// peBAf[^

	UCSR1B = V_UCSR1B;

	write_data_pos = 0;													// ݎMf[^|WV
	data_get_run = CODE_DATA_GET_END;									// Mf[^擾I
	rxd_rec = 0;														// qwO͂qwPM
}

//***************************
// UART1 RXD Interrupt
//***************************
ISR	(USART1_RX_vect)
{
	uart1_err_receiver = (UCSR1A & ~0x01);										// G[Ԏ擾iŉʃrbgOFqwOj
	uart1_err_receiver <<= 8;
	uart1_err_receiver |= UDR1;													// Mf[^擾

	if(save_set_pass_time == CODE_PASS_TIME_ON){								// oߎԗL̎
		uart1_pass_time = get_pass_time_with_clear();							// oߎԎ擾iNA[tj
	}

	if(data_get_run == CODE_DATA_GET_WAIT){										// gKf[^҂
		if((uart1_err_receiver & 0x00ff)==trig_data){							// gKf[^ƈv
			if(set_end_less == CODE_TRIG_CYCL){									// X^[ggK{̎
				set_end_less = CODE_ONE_CYCL;									// ɂ
			}else{
				set_end_less = CODE_END_LESS;									// GhXɂ
			}
			data_get_run = CODE_DATA_GET_RUN;									// Mf[^擾ɂ
		}
	}

	if(data_get_run == CODE_DATA_GET_RUN){										// Mf[^擾
		if(save_set_pass_time == CODE_PASS_TIME_ON){							// oߎԗL̎
			rxd_err_data[write_data_pos] = uart1_pass_time;						// oߎԕۑ
			write_data_pos++;
			write_data_pos &= DATA_POS_MASK;
		}
		rxd_err_data[write_data_pos] = (uart1_err_receiver | 0x0100);			// G[ԂƎMf[^擾iŉʃrbgOFqwPj
		write_data_pos++;
		write_data_pos &= DATA_POS_MASK;
		if(((set_end_less == CODE_ONE_CYCL)&&((N_of_RXD - 1) <= n_of_receive))											// Ńf[^obt@tɂȂA
			 || ((set_end_less == CODE_END_TRIG) && ((uart1_err_receiver & 0x00ff)==trig_data))							// ̓gKGhŃgKf[^̎
			 || ((set_end_less == CODE_ERR_END) && ((uart1_err_receiver & ((1 << (UPE1 + 8))|(1 << (FE1 + 8))))!=0)) ){	// ̓G[GhŃG[L
			data_get_run = CODE_DATA_GET_END;
			pass_time_stop();													// Passage Time stop
			disp_end_pos = 	((write_data_pos - 1) & DATA_POS_MASK);
			if(n_of_receive < n_of_disp_data){									// \܂ŎMĂȂ
				first_pos_num = 1;
			}else if( n_of_receive < N_of_RXD){									// Mf[^Mf[^obt@ȉ̎
				first_pos_num = n_of_receive - (n_of_disp_data - 2);
			}else{																// Mf[^obt@ȏ̎
				first_pos_num = N_of_RXD - (n_of_disp_data - 1);
			}
		}
		rxd_rec = 1;															// M
		tim_led = 2;															// kdc_
		if(n_of_receive < 999){
			n_of_receive++;														// 擾Mf[^{P
		}
	}
}

#endif	//NON_COMM2
