//*****************************************
// AVR ATmega644P Geiger Counter
// CPU Clock:8MHz,16NHz,18.432MHz or 20MHz(Compile Option)
// File Name:Twi.c
//  Use UART0
// KCK[JE^[p
//   iq[Yf[^@is`fgpȂAȂAwIV[^(8MHz)Aɂj
//   i`FbN͂rohdmj
//
//  PC0(SCL): TWI-SCL
//  PC1(SDA): TWI-SDA
//
//  C:\WinAVR-20100110\doc\avr-libc\avr-libc-user-manual.pdf ɊeCN[ht@C̐
//
//  2011/09/08 New  Taishi ATmega644P(J308)
//  2011/10/22 Modify for LND712 by  Taishi
//  2011/10/28 Modify for SBM-20 by  Taishi
//  2011/11/08 Modify for LND712 by  Taishi
//  2011/11/12 Modify for SBM-20 by  Taishi
//  2011/11/12 Modify for J-308  by  Taishi
//  2011/11/24 Modify for All Model  Taishi
//*****************************************

#include	<avr/io.h>
#include	<compat/twi.h>

#include	"AVR_ATmega644_Geiger.h"
#include	"Mega_Twi.h"

#define MAX_TRIES 50

unsigned char	gTwi_status;


#ifdef		TWI_DEBUG_ENABLE
// fobOpǂݏf[^
static unsigned char	gTwi_TWDR_Write[16];	// for Debug
static unsigned char	gTwi_TWDR_Write_pos;	// for Debug
static unsigned char	gTwi_TWDR_Read[16];		// for Debug
static unsigned char	gTwi_TWDR_Read_pos;		// for Debug
static unsigned char	gTwi_TWDR_WStat[16];	// for Debug
static unsigned char	gTwi_TWDR_WStat_pos;	// for Debug

// ************************************
// TWI fobOp݃f[^̎o
//   @cp:PU̃f[^鏊
//        ߂̕@Âق֋߂ɓ
//        cp+0 ߂ŁAcp+15 ԌÂ
//  ߂l Ȃ
// ************************************
void	twi_debug_Load_WriteData(unsigned char *cp)
{
	unsigned char	uc;
	unsigned char	i;

	uc = gTwi_TWDR_Write_pos;
	for(i=0;i<16;i++){
		uc--;
		gTwi_TWDR_Write_pos &= 0x0f;
		*(cp + i) = gTwi_TWDR_Write[uc];
	}
}
// ************************************
// TWI fobOpǂݏof[^̎o
//   @cp:PU̃f[^鏊
//        ߂̕@Âق֋߂ɓ
//        cp+0 ߂ŁAcp+15 ԌÂ
//  ߂l Ȃ
// ************************************
void	twi_debug_Load_ReadData(unsigned char *cp)
{
	unsigned char	uc;
	unsigned char	i;

	uc = gTwi_TWDR_Read_pos;
	for(i=0;i<16;i++){
		uc--;
		gTwi_TWDR_Read_pos &= 0x0f;
		*(cp + i) = gTwi_TWDR_Read[uc];
	}
}
// ************************************
// TWI fobOpݎԃf[^̎o
//   @cp:PU̃f[^鏊
//        ߂̕@Âق֋߂ɓ
//        cp+0 ߂ŁAcp+15 ԌÂ
//  ߂l Ȃ
// ************************************
void	twi_debug_Load_WStatData(unsigned char *cp)
{
	unsigned char	uc;
	unsigned char	i;

	uc = gTwi_TWDR_WStat_pos;
	for(i=0;i<16;i++){
		uc--;
		gTwi_TWDR_WStat_pos &= 0x0f;
		*(cp + i) = gTwi_TWDR_WStat[uc];
	}
}
// ************************************
// TWI fobOpf[^̏ݒ
//   @Ȃ
//  ߂l Ȃ
// ************************************
static
void	twi_debug_data_init(void)
{
	unsigned char	i;

	for(i=0;i<16;i++){
		gTwi_TWDR_Write[i] = 0xff;
		gTwi_TWDR_Read[i] = 0xff;
		gTwi_TWDR_WStat[i] = 0xff;
	}

	gTwi_TWDR_Write_pos = 0;
	gTwi_TWDR_Read_pos = 0;
	gTwi_TWDR_WStat_pos = 0;

}

// ************************************
// TWI fobOp݃f[^̕ۑ
//   @Ȃ
//  ߂l Ȃ
// ************************************
static
void	twi_debug_Save_WriteData(unsigned char d)
{
	gTwi_TWDR_Write[gTwi_TWDR_Write_pos] = d;

	gTwi_TWDR_Write_pos++;
	gTwi_TWDR_Write_pos &= 0x0f;
}

// ************************************
// TWI fobOpǂݏof[^̕ۑ
//   @Ȃ
//  ߂l Ȃ
// ************************************
static
void	twi_debug_Save_ReadData(unsigned char d)
{
	gTwi_TWDR_Read[gTwi_TWDR_Read_pos] = d;

	gTwi_TWDR_Read_pos++;
	gTwi_TWDR_Read_pos &= 0x0f;
}
// ************************************
// TWI fobOpݎԃf[^̕ۑ
//   @Ȃ
//  ߂l Ȃ
// ************************************
static
void	twi_debug_Save_WStatData(unsigned char d)
{
	gTwi_TWDR_WStat[gTwi_TWDR_WStat_pos] = d;

	gTwi_TWDR_WStat_pos++;
	gTwi_TWDR_WStat_pos &= 0x0f;
}
#endif		//TWI_DEBUG_ENABLE

// ************************************
// TWI ݒ
//   @Ȃ
//  ߂l Ȃ
// ************************************
void	twi_init(void)
{
	TWBR = 0xff;						// NbN߂
										// TWSR ̕ 10 1/16Ƃ̂
										// botNbNWlg̎́@SgɂȂ

#ifdef		TWI_DEBUG_ENABLE
	twi_debug_data_init();				// TWI fobOpf[^̏ݒ
#endif		//TWI_DEBUG_ENABLE
}

// ************************************
// TWI ԗv
//   typeFTWI_START, TWI_DATA, TWI_DATA_ACK or TWI_STOP
//  ߂lFTWI (TWSR D3--D7)
//  ʂ҂Ĕ
// ************************************
static
unsigned char twi_transmit(unsigned char type)
{
	switch(type) {
	  case TWI_START: 										// Jnԗv
		TWCR = ( (1 << TWINT) | (1 << TWSTA) | (1 << TWEN) );
		break;
	  case TWI_DATA: 										// ŏ
		TWCR = ( (1 << TWINT) | (1 << TWEN) );
		break;
	  case TWI_DATA_ACK: 									// Lŏ
		TWCR = ( (1 << TWEA) | (1 << TWINT) | (1 << TWEN) );
		break;
	  case TWI_STOP: 										// ~ԗv
		TWCR = ( (1 << TWINT) | (1 << TWEN) | (1 << TWSTO) );
		return(0);
	}

#ifdef	_SIMURATION_					// V~[V̎
		;
#else	//_SIMURATION_

	while (!(TWCR & (1 << TWINT))){							// TWINT-bit PɂȂ܂ő҂
		;
	}

#endif	//_SIMURATION_

	return (TWSR & 0xF8);									// TWI Ԃ߂
}

// ************************************
// TWI Jn
//     dev_id: foCXhc
//       dev_addr: foCXAhX
//        rw_type: TW_WRITE or TW_READ
//  ߂lF0= JnԂɂȂB 1=JnԂɂȂȂ
//
//  ʂ҂Ĕ
// ************************************
unsigned char	twi_start(unsigned int dev_id, unsigned int dev_addr, unsigned char rw_type)
{
	unsigned char	n;
	unsigned char	r_val = 1;

	for(n = 0; n < MAX_TRIES; n++){
		gTwi_status=twi_transmit(TWI_START);								// Jnԗv
		if (gTwi_status == TW_MT_ARB_LOST){									// arbitration lost in SLA+W or data ̎
			continue;														// gC
		}
		if ( ((gTwi_status != TW_START) && (gTwi_status != TW_REP_START)) ){	// start condition transmitted ŖāA repeated start condition transmitted ł
			break;
		}
		TWDR = ( (dev_id & 0xF0) | (dev_addr & 0x0E) | rw_type);			// ǂ  Ɂ@foCXhcƃfoCXAhXt f[^WX^ɃZbg
#ifdef		TWI_DEBUG_ENABLE
		twi_debug_Save_WriteData(TWDR);
#endif		//TWI_DEBUG_ENABLE

		gTwi_status=twi_transmit(TWI_DATA);									//  Mv

#ifdef		TWI_DEBUG_ENABLE
		twi_debug_Save_WStatData(gTwi_status | 0x01);
#endif		//TWI_DEBUG_ENABLE

		if(rw_type == TW_WRITE){
			if ( ((gTwi_status == TW_MT_SLA_NACK) || (gTwi_status == TW_MT_ARB_LOST)) ){	// SLA+W transmitted, NACK received A arbitration lost in SLA+W or data ̎
				continue;														// gC
			}

			if (gTwi_status != TW_MT_SLA_ACK){									// SLA+W transmitted, ACK received Ŗ
				;
			}else{
				r_val=0;														// ߂lOɂ
			}
			break;																// [v𔲂
		}else{
			if ( ((gTwi_status == TW_MR_SLA_NACK) || (gTwi_status == TW_MR_ARB_LOST)) ){	// SLA+R transmitted, NACK received A arbitration lost in SLA+R or data ̎
				continue;														// gC
			}

			if (gTwi_status != TW_MR_SLA_ACK){									// SLA+R transmitted, ACK received Ŗ
				;
			}else{
				r_val=0;														// ߂lOɂ
			}
			break;																// [v𔲂
		}
	}

	return(r_val);
}

// ************************************
// TWI ~
//     Ȃ
//  ߂l Ȃ
//
//  ʂ҂Ĕ
// ************************************
void twi_stop(void)
{
	gTwi_status=twi_transmit(TWI_STOP);	// ~v
}

// ************************************
// TWI 
//     data: ݃f[^
//  ߂lF0= ߂B 1=߂Ȃ
//
//  ʂ҂Ĕ
// ************************************
unsigned char	twi_write(unsigned char data)
{
	unsigned char	r_val = 1;

	TWDR = data;							// ݃f[^ f[^WX^ɃZbg
#ifdef		TWI_DEBUG_ENABLE
	twi_debug_Save_WriteData(TWDR);
#endif		//TWI_DEBUG_ENABLE

	gTwi_status=twi_transmit(TWI_DATA);		//  Mv

#ifdef		TWI_DEBUG_ENABLE
	twi_debug_Save_WStatData(gTwi_status | 0x02);
#endif		//TWI_DEBUG_ENABLE

	if (gTwi_status == TW_MT_DATA_ACK){		// data transmitted, ACK received ̎
		r_val=0;
	}
	return(r_val);
}

// ************************************
// TWI ǂݏo
//     data: ǂݏof[^鏊
//     ack_type: 1:LA 0:
//  ߂lF0= ǂݏoB 1=ǂݏoȂ
//
//  ʂ҂Ĕ
// ************************************
unsigned char	twi_read(unsigned char *data,char ack_type)
{
	unsigned char	r_val = 1;

	if (ack_type) {										// L w̎
		gTwi_status=twi_transmit(TWI_DATA_ACK);			// L f[^ǂݏo
		if (gTwi_status == TW_MR_DATA_ACK){				// data transmitted, ACK received ̎
			*data=TWDR;									// f[^ǂݏoĎw̏ɓ
#ifdef		TWI_DEBUG_ENABLE
			twi_debug_Save_ReadData(*data);
#endif		//TWI_DEBUG_ENABLE
			r_val=0;
		}
	} else {
		gTwi_status=twi_transmit(TWI_DATA);				//  f[^ǂݏo
		if (gTwi_status == TW_MR_DATA_NACK){				// data transmitted, NACK received ̎
			*data=TWDR;									// f[^ǂݏoĎw̏ɓ
#ifdef		TWI_DEBUG_ENABLE
			twi_debug_Save_ReadData(*data);
#endif		//TWI_DEBUG_ENABLE
			r_val=0;
		}
	}

	return(r_val);
}

